/* $Id: ViewerProxySettings.java 1760 2011-02-09 10:37:24Z svc $
 * $Revision: 1760 $
 * $Date: 2011-02-09 11:37:24 +0100 (Wed, 09 Feb 2011) $
 * $Author: svc $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2010 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package dk.netarkivet.viewerproxy;

import dk.netarkivet.common.utils.Settings;

/**
 * Settings specific to the viewerproxy module of NetarchiveSuite.
 */
public class ViewerProxySettings {
    /** The default place in classpath where the settings file can be found. */
    private static final String DEFAULT_SETTINGS_CLASSPATH
            = "dk/netarkivet/viewerproxy/settings.xml";

    /*
     * The static initialiser is called when the class is loaded.
     * It will add default values for all settings defined in this class, by
     * loading them from a settings.xml file in classpath.
     */
    static {
        Settings.addDefaultClasspathSettings(
                DEFAULT_SETTINGS_CLASSPATH
        );
    }

    // NOTE: The constants defining setting names below are left non-final on
    // purpose! Otherwise, the static initialiser that loads default values
    // will not run.

    /**
     * <b>settings.viewerproxy.baseDir</b>: <br> 
     * The main directory for the ViewerProxy, used for storing the Lucene
     * index for the jobs being viewed. This directory can be used by multiple
     * ViewerProxy applications running on the same machine. */
    public static String VIEWERPROXY_DIR = "settings.viewerproxy.baseDir";
    
    /**
     * <b>settings.viewerproxy.tryLookupUriAsFtp</b>: <br>
     * If we fail to lookup an URI, we will try changing the protocol to ftp,
     * if this setting is set to true. The default is false.
     *  
     */
    public static String TRY_LOOKUP_URI_AS_FTP = "settings.viewerproxy.tryLookupUriAsFtp";
    
}
